"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.startLiskCore = exports.isLiskCoreV3Running = exports.installPM2 = exports.installLiskCore = void 0;
const tslib_1 = require("tslib");
const fs = tslib_1.__importStar(require("fs-extra"));
const path = tslib_1.__importStar(require("path"));
const os_1 = require("os");
const fs_extra_1 = require("fs-extra");
const process_1 = require("./process");
const network_1 = require("./network");
const client_1 = require("../client");
const constants_1 = require("../constants");
const fs_1 = require("./fs");
const INSTALL_LISK_CORE_COMMAND = 'npm i -g lisk-core@^4.0.0-rc.1';
const INSTALL_PM2_COMMAND = 'npm i -g pm2';
const PM2_FILE_NAME = 'pm2.migrator.config.json';
const PM2_COMMAND_START = `pm2 start ${PM2_FILE_NAME}`;
const LISK_V3_BACKUP_DATA_DIR = `${(0, os_1.homedir)()}/.lisk/lisk-core-v3`;
const installLiskCore = async () => (0, process_1.execAsync)(INSTALL_LISK_CORE_COMMAND);
exports.installLiskCore = installLiskCore;
const installPM2 = async () => (0, process_1.execAsync)(INSTALL_PM2_COMMAND);
exports.installPM2 = installPM2;
const isLiskCoreV3Running = async (liskCorePath) => {
    try {
        const client = await (0, client_1.getAPIClient)(liskCorePath, true);
        const nodeInfo = await client.node.getNodeInfo();
        return !!nodeInfo;
    }
    catch (_) {
        return false;
    }
};
exports.isLiskCoreV3Running = isLiskCoreV3Running;
const backupDefaultDirectoryIfExists = async (_this, liskCoreV3DataPath) => {
    if ((0, fs_extra_1.existsSync)(liskCoreV3DataPath)) {
        if (!liskCoreV3DataPath.includes('.lisk/lisk-core')) {
            fs.mkdirSync(`${(0, os_1.homedir)()}/.lisk`, { recursive: true });
        }
        _this.log(`Backing Lisk Core v3 data directory at ${liskCoreV3DataPath}`);
        (0, fs_extra_1.renameSync)(liskCoreV3DataPath, LISK_V3_BACKUP_DATA_DIR);
        _this.log(`Backed Lisk Core v3 data directory to: ${LISK_V3_BACKUP_DATA_DIR}`);
    }
};
const copyLegacyDB = async (_this) => {
    _this.log(`Copying the v3.x snapshot to legacy.db at ${constants_1.LEGACY_DB_PATH}`);
    await (0, fs_1.copyDir)(path.resolve(LISK_V3_BACKUP_DATA_DIR, constants_1.SNAPSHOT_DIR), (0, fs_1.resolveAbsolutePath)(constants_1.LEGACY_DB_PATH));
    _this.log(`Legacy database for Lisk Core v4 has been created at ${constants_1.LEGACY_DB_PATH}`);
};
const getFinalConfigPath = async (outputDir, network) => (await (0, fs_1.exists)(`${outputDir}/config.json`))
    ? outputDir
    : path.resolve(__dirname, '../..', 'config', network);
const startLiskCore = async (_this, liskCoreV3DataPath, _config, network, outputDir) => {
    var _a, _b, _c, _d;
    const isCoreV3Running = await (0, exports.isLiskCoreV3Running)(liskCoreV3DataPath);
    if (isCoreV3Running)
        throw new Error('Lisk Core v3 is still running.');
    const networkPort = (_b = (_a = _config === null || _config === void 0 ? void 0 : _config.network) === null || _a === void 0 ? void 0 : _a.port) !== null && _b !== void 0 ? _b : constants_1.DEFAULT_PORT_P2P;
    if (!(await (0, network_1.isPortAvailable)(networkPort))) {
        throw new Error(`Port ${networkPort} is not available for P2P communication.`);
    }
    const rpcPort = (_d = (_c = _config === null || _config === void 0 ? void 0 : _config.network) === null || _c === void 0 ? void 0 : _c.port) !== null && _d !== void 0 ? _d : constants_1.DEFAULT_PORT_RPC;
    if (!(await (0, network_1.isPortAvailable)(rpcPort))) {
        throw new Error(`Port ${rpcPort} is not available to start the RPC server.`);
    }
    await backupDefaultDirectoryIfExists(_this, liskCoreV3DataPath);
    await copyLegacyDB(_this);
    _this.log('Installing pm2...');
    await (0, exports.installPM2)();
    _this.log('Finished installing pm2.');
    _this.log(`Creating PM2 config at ${process.cwd()}/${PM2_FILE_NAME}`);
    const configPath = await getFinalConfigPath(outputDir, network);
    fs.writeFileSync(PM2_FILE_NAME, JSON.stringify({
        name: 'lisk-core-v4',
        script: `lisk-core start --network ${network} --config ${configPath}/config.json`,
    }, null, '\t'));
    _this.log(`Successfully created the PM2 config at ${process.cwd()}/${PM2_FILE_NAME}`);
    _this.log(await (0, process_1.execAsync)(PM2_COMMAND_START));
};
exports.startLiskCore = startLiskCore;
//# sourceMappingURL=node.js.map